//
// Copyright (c) Microsoft. All rights reserved.
// See https://aka.ms/azai/vision/license for the full license information.
//

#pragma once
#include <vision_api_cxx_image_analysis_result.h>

namespace Azure {
namespace AI {
namespace Vision {
namespace ImageAnalysis {

/// <summary>
/// Represents additional information related to an image analysis result.
/// </summary>
/// <remarks>
/// This includes the raw JSON response from the service, and other
/// details related to the service connection and image source.
/// </remarks>
class ImageAnalysisResultDetails
{
public:
    /// <summary>
    /// Return an instance of the ImageAnalysisResultDetails from a given
    /// ImageAnalysisResult object. This is the only way to construct 
    /// the ImageAnalysisResultDetails object.
    /// </summary>
    /// <param name="result">The image analysis result</param>
    /// <returns>
    /// An instance of ImageAnalysisResultDetails containing the result details
    /// </returns>
    static std::shared_ptr<ImageAnalysisResultDetails> FromResult(const std::shared_ptr<ImageAnalysisResult>& result)
    {
        auto ptr = new ImageAnalysisResultDetails(result);
        return std::shared_ptr<ImageAnalysisResultDetails>(ptr);
    }

    /// <summary>
    /// Gets a unique identifier from the Vision Service, associated with this result.
    /// </summary>
    /// <returns>
    /// The result ID string.
    /// </returns>
    std::string GetResultId() const
    {
        return m_parentResult->GetResultId();
    }

    /// <summary>
    /// Gets a unique identifier from the Vision Service, associated with this result.
    /// </summary>
    /// <returns>
    /// The unique ID string.
    /// </returns>
    template<class T = std::string> 
    AI::Core::Details::enable_if_w_or_string_t<T> GetResultId() const
    {
        return m_parentResult->GetResultId<T>();
    }

    /// <summary>
    /// Gets the JSON response payload from the Vision Service that was deserialized
    /// to create the provided ImageAnalysisResult.
    /// </summary>
    /// <returns>
    /// A JSON string.
    /// </returns>
    std::string GetJsonResult()
    { 
        return m_parentResult->Properties.Get(ImageAnalysisResultProperty::Json);
    }

    /// <summary>
    /// Gets the identifier of the analyzed image. This could be the full-path
    /// image file name or the image URL, depending on how the VisionSource was created.
    /// </summary>
    /// <returns>
    /// The image ID string.
    /// </returns>
    std::string GetImageId() const
    { 
        return m_parentResult->Properties.Get("result.image.id");
    }

    /// <summary>
    /// Gets the full URL used to connect to the Image Analysis service
    /// to get these results. It includes the query URL parameters.
    /// </summary>
    /// <returns>
    /// The connection URL.
    /// </returns>
    std::string GetConnectionUrl() const
    { 
        return m_parentResult->Properties.Get("service.connection.url");
    }

private:
    explicit ImageAnalysisResultDetails(const std::shared_ptr<ImageAnalysisResult>& result)
    {
        m_parentResult = result;
    }

    AZAC_DISABLE_COPY_AND_MOVE(ImageAnalysisResultDetails);

    std::shared_ptr<ImageAnalysisResult> m_parentResult;
};

} } } } // Azure::AI::Vision::ImageAnalysis
